/**!
* tippy.js v5.0.0-alpha.0
* (c) 2017-2019 atomiks
* MIT License
*/
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var __chunk_1 = require('../../cjs/chunk-aff5aec0.js');
require('popper.js');

/**
 * Creates a delegate instance that controls the creation of tippy instances
 * for child elements (`target` CSS selector).
 * Port of v4's `target` option to a separate function.
 */
function delegate(targets, options) {
  if (process.env.NODE_ENV !== "production") {
    if (!options || !options.target) {
      throw new Error('[tippy.js ERROR] You must specify a `target` option ' + 'indicating the CSS selector string matching the target elements ' + 'that should receive a tippy.');
    }
  }

  var listeners = [];
  var childTippyInstances = [];
  var target = options.target;
  delete options.target;
  var instanceOrInstances = __chunk_1.tippy(targets, options);

  function onTrigger(event) {
    if (event.target) {
      var targetNode = event.target.closest(target);

      if (targetNode) {
        var instance = __chunk_1.tippy(targetNode, __chunk_1._extends({}, options, {
          showOnInit: true
        }));

        if (instance) {
          childTippyInstances = childTippyInstances.concat(instance);
        }
      }
    }
  }

  function on(element, eventType, listener) {
    var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;
    element.addEventListener(eventType, listener, options);
    listeners.push({
      element: element,
      eventType: eventType,
      listener: listener,
      options: options
    });
  }

  function addEventListeners(instance) {
    var reference = instance.reference;
    instance.props.trigger.trim().split(' ').forEach(function (eventType) {
      switch (eventType) {
        case 'mouseenter':
          {
            on(reference, 'mouseover', onTrigger);
            break;
          }

        case 'focus':
          {
            on(reference, 'focusin', onTrigger);
            break;
          }

        case 'click':
          {
            on(reference, 'click', onTrigger);
          }
      }
    });
  }

  function removeEventListeners(listeners) {
    listeners.forEach(function (_ref) {
      var element = _ref.element,
          eventType = _ref.eventType,
          listener = _ref.listener,
          options = _ref.options;
      element.removeEventListener(eventType, listener, options);
    });
    listeners = [];
  }

  function applyMutations(instance) {
    var originalDestroy = instance.destroy;

    instance.destroy = function () {
      var shouldDestroyChildInstances = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;

      if (shouldDestroyChildInstances) {
        childTippyInstances.forEach(function (instance) {
          instance.destroy();
        });
      }

      childTippyInstances = [];
      removeEventListeners(listeners);
      originalDestroy();
    };

    addEventListeners(instance);
    instance.set({
      trigger: 'manual'
    });
  }

  if (instanceOrInstances) {
    if (Array.isArray(instanceOrInstances)) {
      var instances = instanceOrInstances;
      instances.forEach(applyMutations);
    } else {
      var instance = instanceOrInstances;
      applyMutations(instance);
    }
  }

  return instanceOrInstances;
}

/**
 * Re-uses a single tippy element for many different tippy instances.
 * Replaces v4's `tippy.group()`.
 */

function createSingleton(tippyInstances) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {
    delay: 0
  };

  if (process.env.NODE_ENV !== "production") {
    if (!Array.isArray(tippyInstances)) {
      if (!tippyInstances) {
        throw new Error('[tippy.js ERROR] First argument to `createSingleton()` must ' + 'be an array of tippy instances. The passed value was `' + tippyInstances + '`'); // @ts-ignore
      } else if (tippyInstances.reference && tippyInstances.reference._tippy) {
        throw new Error('[tippy.js ERROR] First argument to `createSingleton()` must ' + 'be an *array* of tippy instances. The passed value was a ' + '*single* tippy instance.');
      }
    }
  }

  var singletonInstance = __chunk_1.tippy(document.createElement('div'));
  var showTimeout;
  var hideTimeout;

  function clearTimeouts() {
    clearTimeout(showTimeout);
    clearTimeout(hideTimeout);
  }

  tippyInstances.forEach(function (instance) {
    var _instance$props = instance.props,
        _onShow = _instance$props.onShow,
        _onTrigger = _instance$props.onTrigger,
        _onUntrigger = _instance$props.onUntrigger;
    var originalClearDelayTimeouts = instance.clearDelayTimeouts;

    instance.clearDelayTimeouts = function () {
      originalClearDelayTimeouts();
      clearTimeouts();
    };

    instance.set({
      delay: 0,
      onShow: function onShow(instance) {
        _onShow(instance);

        return false;
      },
      onTrigger: function onTrigger(instance, event) {
        _onTrigger(instance, event);

        var otherOptions = __chunk_1._extends({}, instance.props);

        delete otherOptions.onShow;
        singletonInstance.set(otherOptions);

        singletonInstance.reference.getBoundingClientRect = function () {
          return instance.reference.getBoundingClientRect();
        };

        clearTimeouts();
        showTimeout = setTimeout(function () {
          singletonInstance.show();
        }, __chunk_1.getValue(options.delay, 0, __chunk_1.tippy.defaults.delay));
      },
      onUntrigger: function onUntrigger(instance, event) {
        _onUntrigger(instance, event);

        clearTimeouts();
        hideTimeout = setTimeout(function () {
          singletonInstance.hide();
        }, __chunk_1.getValue(options.delay, 1, __chunk_1.tippy.defaults.delay));
      }
    }); // Ensure the lifecycles functions are updateable

    var originalSet = instance.set;

    instance.set = function (options) {
      // Delay can't be updated
      delete options.delay;
      originalSet(options);
      _onShow = options.onShow || _onShow;
      _onTrigger = options.onTrigger || _onTrigger;
      _onUntrigger = options.onUntrigger || _onUntrigger;
    };
  });
  return singletonInstance;
}

exports.createSingleton = createSingleton;
exports.delegate = delegate;
//# sourceMappingURL=tippy-addons.js.map
